<?php

/**
 * @file
 * CCK Update functions.
 */

/**
 * Delete a CCK field.
 */
function update_api_content_field_delete(&$ret, $type_name, $field_name) {
  module_load_include('inc', 'content', 'includes/content.crud');

  $success = FALSE;
  $message = 'There was a problem deleting field %field_name from node type %type.';

  if (function_exists('content_types')) {
    if (!($type = content_types($type_name))) {
      $message .= " The content type is invalid.";
    }
    else if (!(isset($type['fields'][$field_name]) && $field = $type['fields'][$field_name])) {
      $message .= " The field does not exist in that content type.";
    }
    else if ($field['locked']) {
      $message .= ' The field is locked.';
    }
    else if ($type && $field && content_field_instance_delete($field_name, $type_name)) {
      $success = TRUE;
      $message = 'Successfully removed field %label from node type %type_name.';
    }
  }
  else {
    $message .= ' CCK is not installed.';
  }

  $args = array(
    '%field_name' => $field_name,
    '%type' => $type_name,
    '%label' => $field['widget']['label'] ? $field['widget']['label'] : $field_name,
    '%type_name' => $type['name'] ? $type['name'] : $type,
  );

  $ret[] = array('success' => $success, 'query' => t($message, $args));
}

/**
 * Update a CCK field.
 */
function update_api_content_field_alter(&$ret, $field) {
  module_load_include('inc', 'content', 'includes/content.crud');

  $success = FALSE;
  $message = 'There was a problem importing settings for field %field_name from node type %type_name.';

  if ($field['field_name'] && $field['type_name']) {
    $instances = content_field_instance_read(array('field_name' => $field['field_name'], 'type_name' => $field['type_name']));
    if (count($instances) == 1) {
      $field = array_merge(content_field_instance_collapse($instances[0]), $field);

      if (content_field_instance_update($field)) {
        $success = TRUE;
        $message = 'Successfully imported field %field_name.';
      }
    }
    else {
      $message .= ' Check that your field_name and type_name settings are correct.';
    }
  }

  $args = array(
    '%field_name' => $field['field_name'],
    '%type_name' => $field['type_name'],
    '%label' => $field['label'] ? $field['label'] : $field['type_name'],
  );

  $ret[] = array('success' => $success, 'query' => t($message, $args));
}

/**
 * Import a CCK exported content type.
 */
function update_api_content_import(&$ret, $content, $type = NULL) {
  $type = is_null($type) ? $content['type']['type'] : $type;

  if ($type && $content) {
    if (function_exists('content_copy_import_form')) {
      $ret[] = _update_api_content_import($content, $type);
    }
    else {
      // Let's try to enable the content_copy module.
      module_enable(array('content_copy'));
  
      $ret[] = _update_api_content_import($content, $type);
  
      // Now disable content_copy.
      module_disable(array('content_copy'));
    }
  }
  else {
    $ret[] = array('success' => FALSE, 'query' => t('Failed to import the content type.  The passed arguments are invalid.'));
  }
}

/**
 * Helper function for update_api_content_import().
 */
function _update_api_content_import($content, $type) {
  if (function_exists('content_copy_import_form')) {
    $form_state['values']['type_name'] = $type;
    $form_state['values']['macro'] = '$content = '. var_export($content, TRUE) .';';
  
    drupal_execute('content_copy_import_form', $form_state);
    return array('success' => TRUE, 'query' => t('Importing of content type %type was attempted.', array('%type' => $content['type']['type'])));
  }
  else {
    return array('success' => FALSE, 'query' => t('Failed to enable the content_copy module for importing type %type.', array('%type' => $type)));
  }
}
