<?php

/**
 * @file
 * Features module update functions.
 */

/**
 * Install features 
 *
 * @param $ret
 *   An array with the results of the hook_update_N() call.
 * @param $features
 *   An array of machine names of features to install.
 */
function update_api_features_install(&$ret, $features = array()) {
  // Load export functions to use in comparison.
  module_load_include('inc', 'features', 'features.export');

  $message = t('All specified features and dependent modules are already enabled.');
  $success = FALSE;

  if (!empty($features) && function_exists('_features_export_maximize_dependencies')) {
    $install = array();
    $disable = array();

    foreach ($features as $name) {
      $feature = feature_load($name, TRUE);
      // Enable feature
      if (!module_exists($name)) {
        if (!empty($feature->info['dependencies'])) {
          $dependencies = _features_export_maximize_dependencies($feature->info['dependencies']);
          foreach ($dependencies as $dependency) {
            if (!module_exists($dependency)) {
              $install[] = $dependency;
            }
          }
        }
        $install[] = $name;
      }
    }

    if (!empty($install)) {
      $install = array_unique($install);
      module_enable($install);

      // Make sure the install API is available.
      include_once './includes/install.inc';
      drupal_install_modules($install);

      $args = array('!features' => implode(', ', $install));
      $message = format_plural(count($install), "Successfully enabled the !features feature.", "Successfully enabled these features & dependent modules: !features", $args);
      $success = TRUE;
    }
  }

  $ret[] = array('success' => $success, 'query' => $message);
}

/**
 * Revert features
 *
 * @param $ret
 *   An array with the results of the hook_update_N() call.
 * @param $features
 *   An array of machine names of features.
 */
function update_api_features_revert(&$ret, $features = array()) {
  if (function_exists('features_include') && !empty($features)) {
    features_include();
    module_load_include('inc', 'features', 'features.export');
    foreach($features as $module) {
      $feature = feature_load($module, TRUE);
      $overrides = features_detect_overrides($feature);
      if (!empty($overrides)) {
        foreach (array_keys($overrides) as $component) {
          if (features_hook($component, 'features_revert')) {
            $result = features_invoke($component, 'features_revert', $module);
            features_set_signature($module_name, $component);
            if ($result !== FALSE) {
              $result = TRUE;
              $message = t('Reverted all <strong>!component</strong> components for <strong>!module</strong>.', array('!component' => $component, '!module' => $module));
            }
            else {
              $message = t('An error occurred trying to revert <strong>!component</strong> components for <strong>!module</strong>.', array('!component' => $component, '!module' => $module));
            }
            $ret[] = array('success' => $result, 'query' => $message);
          }
        }
      }
    }
  }
}
