<?php

/**
 * @file
 * Imagecache module update functions.
 */

/**
 * Flush out imagecache presets.
 *
 * @param $ret
 *   An array with the results of the hook_update_N() call.
 * @param $presets
 *   An array of preset names of presets to flush. If NULL, all presets will be 
 *   flushed.
 */
function update_api_imagecache_presets_flush(&$ret, $presets = NULL) {
  if (!isset($presets)) {
    $presets = imagecache_presets();
  }

  foreach ($presets as $preset) {
    // Make sure we have an imagecache preset array. We have to ensure it's an array 
    // first, b/c isset($preset['presetname']) returns TRUE for strings.
    if (is_array($preset) && isset($preset['presetname'])) {
      $name = $preset['presetname'];
    }
    else {
      $name = $preset;
      $preset = imagecache_preset_by_name($name);
    }

    // Check to make sure we've got a valid preset and then do the dirty work.
    if (is_array($preset) && $preset) {
      imagecache_preset_flush($preset);
      $ret[] = array('success' => TRUE, 'query' => t('Flushed the <strong>@name</strong> imagecache preset.', array('@name' => $name)));
    }
    else {
      $ret[] = array('success' => FALSE, 'query' => t('Unable to find the <strong>@name</strong> imagecache preset.<br /> @vardump', array('@name' => $name, '@vardump' => var_export($preset, TRUE))));
    }
  }
}