<?php

/**
 * @file
 * System tables update functions.
 */

/**
 * Rename variables
 *
 * @param $vars
 *   An array of variables, each key is the old name, and value is the new name.
 */
function update_api_variable_rename(&$ret, $vars = array()) {
  foreach ($vars as $old => $new) {
    $ret[] = update_sql("UPDATE {variable} SET name = '$new' WHERE name = '$old'");
  }
}

/**
 * Sets variables from cutting and pasting a sql dump of the vars table.
 * Right now, this only accepts arrays for each variable.
 */
function update_api_variables_sql_set(&$ret, $vars = array()) {
  foreach($vars as $var) {
    if (is_array($var)) {
      $name = $var[0];
      $val = unserialize($var[1]);
      variable_set($name, $val);
      $ret[] = array('success' => TRUE, 'query' => t('Updated variable %name', array('%name' => $name)));
    }
    else {
      $output = var_export($var, TRUE);
      $ret[] = array('success' => FALSE, 'query' => "Failed to update the variable.  Here's the contents: $output");
    }
  }
}

/**
 * Sets an array of variables, keyed by the variable name.
 */
function update_api_variables_set(&$ret, $vars = array()) {
  foreach($vars as $name => $val) {
    variable_set($name, $val);
    $ret[] = array('success' => TRUE, 'query' => t('Updated variable %name', array('%name' => $name)));
  }
}

/**
 * Deletes an array of variables.
 */
function update_api_variables_del(&$ret, $vars = array()) {
  foreach($vars as $name) {
    variable_del($name);
    $ret[] = array('success' => TRUE, 'query' => t('Deleted variable %name', array('%name' => $name)));
  }
}

/**
 * Disables the specified modules.
 *
 * @param $ret
 *   An array with the results of the hook_update_N() call.
 * @param $modules
 *   An array of modules to disable.
 * @param $error
 *   Boolean to report modules that failed to disable.  Defaults 
 *   to TRUE.
 */
function update_api_modules_disable(&$ret, $modules = array(), $error = TRUE) {
  $failed = array();

  if ($error) {
    foreach($modules as $key => $module) {
      if (!module_exists($module)) {
        $failed[] = $module;
        unset($modules[$key]);
      }
    }
  }

  if ($count = count($modules)) {
    module_disable($modules);
    $args = array('!modules' => implode(', ', $modules));
    $message = format_plural($count, "Successfully disabled the !modules module.", "Successfully uninstalled these modules: !modules", $args);
    $ret[] = array('success' => TRUE, 'query' => $message);
  }

  if ($error && ($count = count($failed))) {
    $args = array('!modules' => implode(', ', $failed));
    $message = format_plural($count, "Unable to disable the !modules module.  It may be non-existent or already disabled.", "Unable to disable these modules: !modules.  They may be non-existent or already disabled.", $args);
    $ret[] = array('success' => FALSE, 'query' => $message);
  }
}

/**
 * Disables and uninstalls the specified modules.
 *
 * @param $ret
 *   An array with the results of the hook_update_N() call.
 * @param $modules
 *   An array of modules to disable and uninstall.
 */
function update_api_modules_uninstall(&$ret, $modules = array()) {
  // Make sure the install API is available.
  include_once './includes/install.inc';

  $uninstalled = $failed = array();

  // Disable the modules, just to be safe.
  update_api_modules_disable($ret, $modules, FALSE);

  foreach($modules as $name) {
    if ($module = _update_api_module_uninstallable($name)) {
      drupal_uninstall_module($name);
      $uninstalled[] = $module->info['name'];
    }
    else {
      $failed[] = $name;
    }
  }

  if ($count = count($uninstalled)) {
    $args = array('!modules' => implode(', ', $uninstalled));
    $message = format_plural($count, "Successfully uninstalled the !modules module.", "Successfully uninstalled these modules: !modules", $args);
    $ret[] = array('success' => TRUE, 'query' => $message);
  }

  if ($count = count($failed)) {
    $args = array('!modules' => implode(', ', $failed));
    $message = format_plural($count, "Unable to uninstall the !modules module.  It may already be uninstalled.", "Unable to uninstall these modules: !modules.  They may already be uninstalled.", $args);
    $ret[] = array('success' => FALSE, 'query' => $message);
  }
}

/**
 * Helper function to determine if a module is uninstallable.
 *
 * @param $name
 *   The machine name of the module in question.
 * @return
 *   The info object of the module if it is uninstallable,
 *   otherwise boolean FALSE.
 */
function _update_api_module_uninstallable($name) {
  static $uninstallables = array();

  if (empty($uninstallables)) {
    // Pull all disabled modules from the system table.
    $disabled_modules = db_query("SELECT name, filename, info FROM {system} WHERE type = 'module' AND status = 0 AND schema_version > %d ORDER BY name", SCHEMA_UNINSTALLED);
    while ($module = db_fetch_object($disabled_modules)) {
  
      // Grab the module info
      $info = unserialize($module->info);
  
      // Load the .install file, and check for an uninstall hook.
      // If the hook exists, the module can be uninstalled.
      module_load_install($module->name);
      if (module_hook($module->name, 'uninstall')) {
        $uninstallables[$module->name] = $module;
      }
    }
  }

  return isset($uninstallables[$name]) ? $uninstallables[$name] : FALSE;
}