<?php

/**
 * @file
 * All User Module related update functions.
 *
 * Most permissions related functions were quickly borrowed from the Permissions 
 * API Module, found at http://drupal.org/project/permissions_api.  @todo These 
 * permissions functions definitely need some love.
 */

/**
 * Rename an array of roles.
 *
 * @param $vars
 *   An array of roles, each key is the old name, and value is the new name.
 */
function update_api_roles_rename(&$ret, $roles) {
  foreach($roles as $old => $new) {
    $ret[] = update_sql("UPDATE {role} SET name = '$new' WHERE name = '$old'");
  }
}

/**
 * Helper function to grant permissions to a role.
 * Code borrowed from permissions_api.
 *
 * @param $role_name
 *   String containing the name of the role to update.
 * @param $new_permissions
 *   An array of permissions to grant to the user.
 */
function update_api_permissions_grant(&$ret, $role_name, $new_permissions) {
  $permissions = array();
  $updated_permissions = '';
  $role = update_api_get_role($role_name);
  if (count($new_permissions) > 0 && $role) {
    // Fetch the permissions string for the given role id
    $permissions = update_api_get_permissions_for_role($role->rid);
    // Check to see if there are existing permissions
    if(count($permissions) > 0) {
      // Add the new permissions if the role doesn't already have the permission
      foreach($new_permissions as $permission) {
        if(!in_array($permission, $permissions)) {
          $permissions[] = trim($permission);
        }
      }

      // rebuild the permission string
      $updated_permissions = implode(', ', $permissions);
      $ret[] = update_sql("UPDATE {permission} SET perm = '$updated_permissions' WHERE rid = {$role->rid}");
    }
    else{
      // No permissions have been set for this role, so we need to insert some
      foreach($new_permissions as $permission){
        $permissions[] = trim($permission);
      }
      // rebuild the permission string
      $updated_permissions = implode(', ', $permissions);
      $ret[] = update_sql("INSERT INTO {permission} (rid, perm, tid) VALUES({$role->rid},'$updated_permissions',0)");
    }
  }
}

/**
 * Helper function to revoke permissions from a role.
 * Code borrowed from permissions_api.
 */
function update_api_permissions_revoke(&$ret, $role_name, $new_permissions) {
  $permissions = array();
  $updated_permissions = '';
  $role = update_api_get_role($role_name);
  if (is_array($new_permissions)) {
    $permissions = update_api_get_permissions_for_role($role->rid);
    if (count($permissions) > 0) {
      // Unset the permission from the array
      foreach($new_permissions as $permission){
        $index = array_search($permission, $permissions);
        if($index >= 0){
          unset($permissions[$index]);       
        }
      }
      
      // rebuild the permission string
      $updated_permissions = implode(', ', $permissions);
      $ret[] = update_sql("UPDATE {permission} SET perm = '$update_permissions' WHERE rid = {$role->rid}");
    }
  }
}

/**
 * Retrieve a role by name.
 */
function update_api_get_role($name) {
  return db_fetch_object(db_query("SELECT * FROM {role} r WHERE r.name = '%s'", $name));
}

/**
 * Return an array of permissions for a given role.
 */
function update_api_get_permissions_for_role($rid){
  if($rid){
    $perm_string = db_result(db_query("SELECT perm FROM {permission} WHERE rid = %d", $rid));
    if (strlen($perm_string) > 0) {
      $permissions = explode(',', $perm_string);
      foreach($permissions as $index => $perm){
        $permissions[$index] = trim($perm);
      }
    }
  }
  return $permissions;
}
